using System;
using System.Runtime.InteropServices;

using MDBHandle = System.UInt32;
using MDBValue = System.UInt32;

namespace Hula {

public class MDB {
	public bool InitLibrary() {
		/* this could be a constructor - need to look up exceptions... */
		return MDBRaw.Init();
	}
	
	public bool DropPriviledge() {
		return (MDBRaw.setEffectiveUser (MDBRaw.getUnpriviledgedUser()) < 0);
	}
	
	private MDBHandle handle;
	
	public bool Authenticate (string ObjectName, string Password, string Module) {
		MDBHandle h = MDBRaw.Authenticate(Module, ObjectName, Password);
		if (h == 0) 
			return false;
		handle = h;
		return true;
	}
	
	public bool Authenticate (string ObjectName, string Password) {
		return Authenticate (ObjectName, Password, "Hula");
	}
	
	public bool ObjectExists (string ObjectName) {
		MDBValue v = MDBRaw.CreateValueStruct(handle, null);
		
		bool result = MDBRaw.IsObject(ObjectName, v);
		return result;
	}
	
	public MDBObject AddObject (string Context, string Name, string Class) {
		MDBValue v = MDBRaw.CreateValueStruct(handle, null);
		
		MDBRaw.SetValueStructContext(Context, v);
		
		MDBValue attribute = MDBRaw.ShareContext(v);
		MDBValue data	   = MDBRaw.ShareContext(v);
		
		if (! MDBRaw.CreateObject (Name, Class, attribute, data, v))
			return null;
		return new MDBObject(Context, Name, handle, v);
	}
	
	public MDBObject GetObject (string Context, string Name) {
		MDBValue v = MDBRaw.CreateValueStruct(handle, null);
		
		MDBRaw.SetValueStructContext(Context, v);
		
		return new MDBObject(Context, Name, handle, v);
	}
	
	public int GetAPIVersion (bool wantCompatible, string description) {
		return MDBRaw.GetAPIVersion(wantCompatible, description, handle);
	}
	
	~MDB() {
		MDBRaw.Shutdown();
	}
}

public class MDBObject {
	private MDBHandle handle;
	private MDBValue valuestruct;
	private string context;
	private string name;
	
	public MDBObject (string Context, string Name, MDBHandle h, MDBValue v) {
		context = Context;
		name = Name;
		handle = h;
		valuestruct = v;
	}
	
	public bool AddAttribute (string Attribute, string Value) {
		return MDBRaw.Add(name, Attribute, Value, valuestruct);
	}
	
	public bool ChangePassword (string oldpassword, string newpassword) {
		return MDBRaw.ChangePasswordEx(name, oldpassword, newpassword, valuestruct);
	}
	
	public bool Delete () {
		return MDBRaw.DeleteObject(name, false, valuestruct);
	}
	
	public bool DeleteRecursive () {
		return MDBRaw.DeleteObject(name, true, valuestruct);
	}
	
	public bool Rename (string NewName) {
		if (! MDBRaw.RenameObject(name, NewName, valuestruct)) 
			return false;
		name = NewName;
		return true;
	}
	
	public bool debug_InternalValue () {
		return MDBRaw.DumpValueStruct (valuestruct);
	}
}

public class MDBRaw {
	const string MDBLib = "hulamdb";
	const string XPLLib = "hulaxpl";
	const string MSGLib = "hulamsgapi";

	/* initialise the MDB library. This fails in two ways:
	 * - library cannot be loaded for some reason
	 * - file permissions (mdbfile) disallow access
	 * Sadly we don't really seem to be able to tell 
	 * the difference!
	 */
	[DllImport(MDBLib,EntryPoint="MDBInit")]
	public static extern bool 	Init();
	
	[DllImport(MDBLib,EntryPoint="MDBShutdown")]
	public static extern bool	Shutdown();
	
	[DllImport(MDBLib,EntryPoint="MDBAuthenticate")]
	public static extern MDBHandle	Authenticate (string Module, string ObjectName, string Password);
	
	[DllImport(MDBLib,EntryPoint="MDBGetServerInfo")]
	public static extern bool	GetServerInfo (string hostDN, string hostTree, MDBValue v);
	
	[DllImport(MDBLib,EntryPoint="MDBCreateValueStruct")]
	public static extern MDBValue	CreateValueStruct (MDBHandle Handle, string Context);
	
	[DllImport(MDBLib,EntryPoint="MDBSetValueStructContext")]
	public static extern bool	SetValueStructContext (string Context, MDBValue v);
	
	[DllImport(MDBLib,EntryPoint="MDBShareContext")]
	public static extern MDBValue	ShareContext (MDBHandle Handle);
	
	[DllImport(MDBLib,EntryPoint="MDBIsObject")]
	public static extern bool	IsObject (string objectName, MDBValue v);
	
	[DllImport(MDBLib,EntryPoint="MDBCreateObject")]
	public static extern bool	CreateObject (string objectName, string Class, MDBValue Attribute, 
						MDBValue Data, MDBValue v);
	
	[DllImport(MDBLib,EntryPoint="MDBRenameObject")]
	public static extern bool	RenameObject (string objectName, string newObjectName, MDBValue v);
	
	[DllImport(MDBLib,EntryPoint="MDBDeleteObject")]
	public static extern bool	DeleteObject (string objectName, bool recurse, MDBValue v);
	
	[DllImport(MDBLib,EntryPoint="MDBAdd")]
	public static extern bool	Add (string objectName, string addtribute, string value, MDBValue v);
	
	[DllImport(MDBLib,EntryPoint="MDBRead")]
	public static extern long	Read (string objectName, string attributeName, MDBValue v);
	
	[DllImport(MDBLib,EntryPoint="MDBChangePasswordEx")]
	public static extern bool	ChangePasswordEx(string objectName, string oldPassword, string newPassword,
						MDBValue v);
	
	/* some debuggy stuff? */
	[DllImport(MDBLib,EntryPoint="MDBDumpValueStruct")]
	public static extern bool	DumpValueStruct(MDBValue v);
	
	[DllImport(MDBLib,EntryPoint="MDBGetAPIVersion")]
	public static extern int	GetAPIVersion(bool wantCompatible, string description, MDBHandle Context);
	
	/* following functions should be in their own library eventually,
	 * they're not really anything to do with hulamdb 
	 */
	[DllImport(MSGLib,EntryPoint="MsgGetUnpriviledgedUser")]
	public static extern string	getUnpriviledgedUser ();
	[DllImport(XPLLib,EntryPoint="XplSetEffectiveUser")]
	public static extern int	setEffectiveUser (string username);
}

}
