// using System;
using System.Security.Cryptography;
using System.Net.Sockets;
using System.Text;
using System.IO;
using System;

namespace Hula {

public class NMAP {
	TcpClient connection;
	StreamWriter writer;
	StreamReader reader;
	
	public struct NMAPResponse {
		public int 	code;
		public string 	text;
	}
	
	private string greeting_salt;
	
	public bool Open(string host, int port) {
		connection = new TcpClient();
		connection.Connect(host, port);
		
		writer = new StreamWriter(connection.GetStream());
		reader = new StreamReader(connection.GetStream());
		
		NMAPResponse r = this.ReadFullReply();
		if (r.code == 1000)
			return true;
		if (r.code == 4242) {
			// 4242 means we have to authenticate, so save the salt
			int salt_start = r.text.IndexOf('<') + 1;
			int salt_length = r.text.IndexOf('>') - salt_start;
			greeting_salt = r.text.Substring(salt_start, salt_length);
			return true;
		}
		return false;
	}
		
	public void Close () {
		WriteRequest("QUIT");
		NetworkStream stream = connection.GetStream();
		stream.Close();
	}
	
	public bool AuthAsUser (string user, string password) {
		this.WriteRequest("PASS USER {0} {1}", user, password);
		if (this.ReadReply()==1000)
			return true;
		return false;
	}
	
	public bool Auth (string password) {
		MD5 md5 = MD5.Create();
		
		UTF8Encoding ue = new UTF8Encoding();
		byte[] pass = ue.GetBytes(greeting_salt + password);
		
		byte[] p = md5.ComputeHash(pass);
		
		// need to do a stringbuild on p to make it ascii
		
		Console.WriteLine("\nUsing '{0}' as password", p.ToString());
		
		this.WriteRequest("PASS SYSTEM {0}", password);
		if (this.ReadReply()==1000)
			return true;
		this.Close(); /* should note we've been booted? */
		return false;
	}
	
	public string NMAPVersion () {
		WriteRequest("NVER");
		NMAPResponse response = ReadFullReply();
		return response.text;
	}
	
	public void WriteRequest (string request, params object [] args) {
		writer.WriteLine(request, args);
		writer.Flush();
	}
	
	public int ReadReply () {	
		NMAPResponse response = this.ReadFullReply();
		
		return response.code;
	}
	
	public NMAPResponse ReadFullReply () {
		NMAPResponse response;
		
		string result = reader.ReadLine();
		response.code = Convert.ToInt32(result.Substring(0,4));
		response.text = result.Substring(5, result.Length-5);
		
		return response;
	}

}

}
